use std::sync::Arc;

use wgpu::{Device, Queue};
use winit::{dpi::PhysicalSize, window::Window};

use crate::{
  game::anm::LoadedFile,
  utils::{game::Game, soon::Soon},
};

use super::Manager;

impl Manager {
  pub fn start_load_anm(
    &mut self,
    device: &Arc<Device>,
    queue: &Arc<Queue>,
    size: PhysicalSize<u32>,
    game: Game,
    file_name: impl Into<String>,
  ) -> Soon<Arc<LoadedFile>> {
    let file_name: String = file_name.into();
    if let Some(loaded_anm) = self.anm_files.get(&file_name) {
      return Soon::Value(loaded_anm.clone());
    }

    let sender = self.anm_sender.clone();
    let (device, queue) = (device.clone(), queue.clone());
    Soon::new(async move {
      let file = Arc::new(LoadedFile::load(device, queue, size, game, &file_name).await);

      sender.send((file_name, file.clone())).unwrap();

      file
    })
  }

  pub(super) fn receive_preloaded_anms(&mut self) {
    while let Ok((file_name, loaded_anm)) = self.anm_receiver.try_recv() {
      self.anm_files.insert(file_name, loaded_anm);
    }
  }

  pub fn load_anm(
    &mut self,
    device: &Arc<Device>,
    queue: &Arc<Queue>,
    window: &Window,
    game: Game,
    file_name: impl Into<String>,
  ) -> Arc<LoadedFile> {
    let file_name = file_name.into();
    if let Some(loaded_anm) = self.anm_files.get(&file_name) {
      return loaded_anm.clone();
    }

    let loaded_anm = Arc::new(async_std::task::block_on(LoadedFile::load(
      device.clone(),
      queue.clone(),
      window.inner_size(),
      game,
      &file_name,
    )));

    self.anm_files.insert(file_name, loaded_anm.clone());
    loaded_anm
  }
}
