use std::{cell::RefCell, rc::Rc};

use vm::{LoadedFile, StageVm};

use crate::utils::game::Game;

use super::GameContext;

mod enemy;
mod vm;

#[derive(Debug, Clone, Copy)]
pub enum StageNumber {
  Stage1 = 1,
  Stage2 = 2,
  Stage3 = 3,
  Stage4 = 4,
  Stage5 = 5,
  Stage6 = 6,
  Extra = 7,
}

impl StageNumber {
  pub fn next(&self) -> Option<Self> {
    match self {
      Self::Stage1 => Some(Self::Stage2),
      Self::Stage2 => Some(Self::Stage3),
      Self::Stage3 => Some(Self::Stage4),
      Self::Stage4 => Some(Self::Stage5),
      Self::Stage5 => Some(Self::Stage6),
      Self::Stage6 => None,
      Self::Extra => None,
    }
  }

  pub fn stage_music(&self, game: Game, boss: bool) -> &'static str {
    match (self, game, boss) {
      (Self::Stage1, Game::Mof, false) => "th10_00.wav",
      (Self::Stage1, Game::Mof, true) => "th10_01.wav",
      _ => todo!()
    }
  }
}

pub struct Stage {
  vm: StageVm,
  enemies: Vec<Rc<RefCell<enemy::Vm>>>,
  game: Game,
  stage_number: StageNumber,
}

impl Stage {
  pub async fn new(context: &mut GameContext<'_>, game: Game, stage_number: StageNumber) -> Self {
    let file = LoadedFile::load(context, game, format!("stage0{}.std", stage_number as u8)).await;

    Self {
      vm: StageVm::new(file),
      enemies: vec![],
      game,
      stage_number,
    }
  }

  pub fn started(&mut self, context: &mut GameContext) {
    context.sound_manager.get_bgm_manager(self.game).play_track(self.stage_number.stage_music(self.game, false));
  }

  pub fn update(&mut self, context: &mut GameContext) {
    self.vm.update(context);
  }
}
