use std::{cell::RefCell, sync::Arc};

use glam::Vec3;
use sfsm::{State, TransitGuard, Transition};

use crate::{
  game::{
    anm::{LoadedFile, Vm, VmLocation},
    GameContext,
  },
  utils::{game::Game, soon::Soon},
};

use super::{gameplay::Gameplay, title_mof::TitleScreenMof, title_sa::TitleScreenSa, UPDATE_CONTEXT};

pub struct Loading {
  _sig: Vm,
  _ascii_loading: Vm,
  to_gameplay: bool,

  title_anm: RefCell<Soon<Arc<LoadedFile>>>,
}

impl Loading {
  pub fn new(context: &mut GameContext) -> Loading {
    let ascii = context.load_anm(Game::Sa, "ascii.anm");
    let sig_anm = context.load_anm(Game::Sa, "sig.anm");
    let sig = context.anm_manager.new_vm(sig_anm, None, 0, VmLocation::new_ui());
    let ascii_loading = context.anm_manager.new_vm(ascii.clone(), None, 16, VmLocation::new_ui());
    ascii_loading.borrow_mut().origin = Vec3::new(480.0, 392.0, 0.0);

    Loading {
      _sig: sig,
      _ascii_loading: ascii_loading,
      to_gameplay: true,

      title_anm: context.start_load_anm(Game::Mof, "title.anm").into(),
    }
  }
}

sfsm::derive_state!(Loading);

impl Transition<TitleScreenSa> for Loading {
  fn guard(&self) -> TransitGuard {
    UPDATE_CONTEXT
      .with(|context| context.sound_manager.is_bgm_manager_loaded() && self.title_anm.borrow_mut().is_done())
      .into()
  }
}

impl Transition<TitleScreenMof> for Loading {
  fn guard(&self) -> TransitGuard {
    UPDATE_CONTEXT
      .with(|context|!self.to_gameplay && context.sound_manager.is_bgm_manager_loaded() && self.title_anm.borrow_mut().is_done())
      .into()
  }
}

impl Transition<Gameplay> for Loading {
  fn guard(&self) -> TransitGuard {
    UPDATE_CONTEXT
      .with(|context| self.to_gameplay && context.sound_manager.is_bgm_manager_loaded() && self.title_anm.borrow_mut().is_done())
      .into()
  }
}
